import log from 'electron-log'
import { autoUpdater } from 'electron-updater'
import type { BrowserWindow } from 'electron'
import { ipcMain } from 'electron'

autoUpdater.logger = log
log.info('App starting...')

type MessageType =
    | 'checking-for-update'
    | 'error'
    | 'update-available'
    | 'update-not-available'
    | 'download-progress'
    | 'update-downloaded'

export async function useUpdater(win: BrowserWindow) {
    function checkForUpdate() {
        autoUpdater.checkForUpdates()
    }

    function sendUpdateMessageToWindow(body: { type: MessageType; text: string }) {
        log.info(`${body.type}: ${body.text}`)
        win.webContents.send('updater-message', body)
    }

    autoUpdater.on('checking-for-update', () => {
        sendUpdateMessageToWindow({ type: 'checking-for-update', text: '正在检查更新...' })
    })
    autoUpdater.on('update-available', (info) => {
        sendUpdateMessageToWindow({ type: 'update-available', text: `发现新版本 v${info.version}` })
    })
    autoUpdater.on('update-not-available', () => {
        sendUpdateMessageToWindow({ type: 'update-not-available', text: '当前已是最新版本！' })
    })
    autoUpdater.on('error', (err) => {
        sendUpdateMessageToWindow({ type: 'error', text: err.message })
    })
    autoUpdater.on('download-progress', (progress) => {
        sendUpdateMessageToWindow({
            type: 'download-progress',
            text: `正在下载新版本 ${Number(progress.percent).toFixed(2)}%`,
        })
    })
    autoUpdater.on('update-downloaded', () => {
        sendUpdateMessageToWindow({ type: 'update-downloaded', text: '下载完成' })

        setTimeout(() => {
            // 退出并安装更新包
            autoUpdater.quitAndInstall(false, true)
            win.destroy()
        }, 1500)
    })

    // 监听界面发送的检查更新事件
    ipcMain.on('checkForUpdate', () => {
        checkForUpdate()
    })

    // 检查更新
    checkForUpdate()
}
